package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.BalanceWeighingDTO;
import digital.laboratory.platform.reagent.dto.StandardMaterialApplicationDTO;
import digital.laboratory.platform.reagent.entity.CatalogueDetails;
import digital.laboratory.platform.reagent.entity.StandardMaterialApplication;
import digital.laboratory.platform.reagent.service.StandardMaterialApplicationService;
import digital.laboratory.platform.reagent.vo.PurchaseCatalogueVO;
import digital.laboratory.platform.reagent.vo.StandardMaterialApplicationVO;
import digital.laboratory.platform.sys.feign.RemoteReagentConsumableService;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * (标准物质领用/归还登记表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质领用 / 归还登记表) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/standard_material_application")
@Api(value = "standard_material_application", tags = "(标准物质领用/归还登记表)管理")
public class StandardMaterialApplicationController {

    private final StandardMaterialApplicationService standardMaterialApplicationService;
    private final OssFile ossFile;

    /**
     * 通过id查询(标准物质领用/归还登记表)
     *
     * @param id, id
     * @return R
     */
    @ApiOperation(value = "通过id/扫码出的编号查询", notes = "通过id/扫码出的编号查询")
    @GetMapping()
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_application_get')")
    public R<StandardMaterialApplicationVO> getById(String id, String number, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (StrUtil.isNotBlank(number)) {

            StandardMaterialApplication reference_material_number = standardMaterialApplicationService.getOne(Wrappers.<StandardMaterialApplication>query().eq(StrUtil.isNotBlank(number), "reference_material_number", number)
                    .eq("status", 0));


            if (reference_material_number != null) {
                StandardMaterialApplicationVO standardMaterialApplicationVO = standardMaterialApplicationService.getStandardMaterialApplicationVO(reference_material_number.getStandardMaterialApplicationId());

                return R.ok(standardMaterialApplicationVO);
            } else return R.failed("没有查询到该标准物质领用/归还的信息");
        }

        StandardMaterialApplicationVO standardMaterialApplicationVO = standardMaterialApplicationService.getStandardMaterialApplicationVO(id);

        return R.ok(standardMaterialApplicationVO);
        //return R.ok(standardMaterialApplicationService.getById(standardMaterialApplicationId));
    }

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @ApiOperation(value = "领用分页查询", notes = "领用分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_application_page')")
    public R<IPage<StandardMaterialApplicationVO>> getStandardMaterialApplicationPage(Page<StandardMaterialApplication> page, String name, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<StandardMaterialApplicationVO> standardMaterialApplicationSList = standardMaterialApplicationService.getPage(page, Wrappers.<StandardMaterialApplication>query()
                .like(StrUtil.isNotBlank(name), "reference_substance_name", name).or()
                .like(StrUtil.isNotBlank(name), "reference_material_number", name)
                .orderByDesc("date_of_claim")
        );
        return R.ok(standardMaterialApplicationSList);
//		return R.ok(standardMaterialApplicationService.page(page, Wrappers.query(standardMaterialApplication)));
    }

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @ApiOperation(value = "归还任务分页查询", notes = "归还任务分页查询")
    @GetMapping("/return/page")
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_application_page')")
    public R<IPage<StandardMaterialApplicationVO>> getList(Page<StandardMaterialApplication> page, String name, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate endTime, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<StandardMaterialApplicationVO> standardMaterialApplicationSList = standardMaterialApplicationService.getList(page, Wrappers.<StandardMaterialApplication>query()
                .eq("status", 0)
                .like(StrUtil.isNotBlank(name), "reference_substance_name", name)
                .ge(startTime != null, "create_time", startTime)
                .le(endTime != null, "create_time", endTime)
                .orderByDesc("create_time")

        );
        return R.ok(standardMaterialApplicationSList);
//		return R.ok(standardMaterialApplicationService.page(page, Wrappers.query(standardMaterialApplication)));
    }

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @ApiOperation(value = "归还记录分页查询", notes = "归还记录分页查询")
    @GetMapping("/return")
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_application_page')")
    public R<IPage<StandardMaterialApplicationVO>> getPage(Page<StandardMaterialApplication> page, String rid, String name, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate endTime, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<StandardMaterialApplicationVO> standardMaterialApplicationSList = standardMaterialApplicationService.getList(page, Wrappers.<StandardMaterialApplication>query()
                .eq("status", 1)
                .eq(StrUtil.isNotBlank(rid), "reference_material_id", rid)
                .like(StrUtil.isNotBlank(name), "reference_substance_name", name)
                .ge(startTime != null, "create_time", startTime)
                .le(endTime != null, "create_time", endTime)
                .orderByDesc("create_time").or()
                .eq("status", 1)
                .eq(StrUtil.isNotBlank(rid), "reference_material_id", rid)
                .like(StrUtil.isNotBlank(name), "reference_material_number", name)
                .ge(startTime != null, "create_time", startTime)
                .le(endTime != null, "create_time", endTime)
                .orderByDesc("create_time")

        );
        return R.ok(standardMaterialApplicationSList);
//		return R.ok(standardMaterialApplicationService.page(page, Wrappers.query(standardMaterialApplication)));
    }


    /**
     * 归还(标准物质领用/归还登记表)
     *
     * @param standardMaterialApplicationDTO (标准物质领用/归还登记表)
     * @return R
     */
    @ApiOperation(value = "归还(标准物质领用/归还登记表)", notes = "归还(标准物质领用/归还登记表)")
    @SysLog("修改(标准物质领用/归还登记表)")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_application_return')")
    public R<StandardMaterialApplication> putUpdateById(@RequestBody StandardMaterialApplicationDTO standardMaterialApplicationDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StandardMaterialApplication standardMaterialApplication = standardMaterialApplicationService.returnById(standardMaterialApplicationDTO);

        if (standardMaterialApplication != null) {
            return R.ok(standardMaterialApplication, "归还成功");
        } else return R.failed("归还失败");
    }

    /**
     * 通过id删除(标准物质领用/归还登记表)
     *
     * @param standardMaterialApplicationId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(标准物质领用/归还登记表)", notes = "通过id删除(标准物质领用/归还登记表)")
    @SysLog("通过id删除(标准物质领用/归还登记表)")
    @DeleteMapping("/{standardMaterialApplicationId}")
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_application_del')")
    public R<StandardMaterialApplication> deleteById(String standardMaterialApplicationId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StandardMaterialApplication oldStandardMaterialApplication = standardMaterialApplicationService.getById(standardMaterialApplicationId);

        if (standardMaterialApplicationService.removeById(standardMaterialApplicationId)) {
            return R.ok(oldStandardMaterialApplication, "对象删除成功");
        } else {
            return R.failed(oldStandardMaterialApplication, "对象删除失败");
        }

    }

    /**-------采购目录打印*/
    /**
     * 采购目录打印
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "标准物质领用归还登记表打印", notes = "标准物质领用归还登记表打印")
    @SysLog("标准物质领用归还登记表打印")
    @PostMapping("/print")
    @PreAuthorize("@pms.hasPermission('reagent_standard_material_application_print')")
    public void bizGetPDFInventory(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) {
        System.out.println("bizApplyWord.................");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        String ids = IdWorker.get32UUID().toUpperCase();

        String[] split = id.split(",");

        List<StandardMaterialApplicationVO> voList = new ArrayList<>();

        for (String s : split) {

            StandardMaterialApplicationVO standardMaterialApplicationVO = standardMaterialApplicationService.getStandardMaterialApplicationVO(s);

            voList.add(standardMaterialApplicationVO);
        }

        if (voList.isEmpty()) {
            throw new RuntimeException("未查询到相关信息");
        }


        String applyFileName = "标准物质领用归还登记表-" + ids;

        String pdfFilePath = "document" + "/" + "standardMaterialApplication" + "/" + ids + "/" + applyFileName + ".pdf";
        try {
            //直接调用pdf方法
            standardMaterialApplicationService.standardMaterialApplicationTablePDF(voList, ids, theHttpServletRequest, httpServletResponse);
            ossFile.fileGet(pdfFilePath, httpServletResponse.getOutputStream());
            httpServletResponse.setContentType(applyFileName);
        } catch (Exception e) {
            System.out.println(String.format("minioFile objectExist() Exception. %s", e.getLocalizedMessage()));
            e.printStackTrace();
        }
    }
    @ApiOperation(value = "天平称量后记录标准物质使用数量", notes = "天平称量后记录标准物质使用数量")
    @SysLog("天平称量后记录标准物质使用数量")
    @PutMapping("/weighing")
    public R<String> getWeighingMass(@RequestBody List<BalanceWeighingDTO> balanceWeighingDTOList) {
        boolean weighingMass = standardMaterialApplicationService.getWeighingMass(balanceWeighingDTOList);
        if (weighingMass) {
            return R.ok("称量成功");
        } else return R.failed("称量失败");
    }


}
